@@ -25,6 +25,7 @@
 from re import compile
 from math import pow, sqrt, pi, e, sin, cos, tan, fmod, log
 from operator import add, sub, mul, truediv, not_, gt, lt, ge, le, eq
+from numpy import irr
 
 Symbol = str
 
@@ -57,54 +58,19 @@ def npv(rate, payments):
     return value
 
 
-def irrResult(values, dates, rate):
-    """IRR helper function.  See irr()"""
-    r = rate + 1
-    result = values[0]
-    for i in range(len(values)):
-        result += values[i] / pow(r, (dates[i] - dates[0]) / 365)
-    return result
-
-
-def irrResultDeriv(values, dates, rate):
-    """IRR helper function - calculates the first derivation.  See irr()"""
-    r = rate + 1
-    result = 0
-    for i in range(len(values)):
-        frac = (dates[i] - dates[0]) / 365
-        result -= frac * values[i] / pow(r, frac + 1)
-    return result
-
-
-def irr(values, guess_rate=0.1):
+def alt_irr(cashflows, guess_rate=1.0):
     """
-    IRR Calculation.  Keep consistent with $.asg.irr() in asg.coffee
-    Source: https://gist.github.com/ghalimi/4597900
+    The IRR or Internal Rate of Return is the annualized effective compounded return rate which can be earned on the
+    invested capital, i.e., the yield on the investment.  This formula is not always accurate.
+
+    http://stackoverflow.com/questions/6892900/what-is-the-numerical-method-used-in-this-implementation-of-irr
     """
+    iterations = 100
+    investment = cashflows[0]
+    for i in range(1, iterations+1):
+        guess_rate *= (1 - npv(guess_rate, cashflows) / investment)
+    return guess_rate
 
-    # Initialize dates and check that values contains at least one positive value and one negative value
-    if max(values) <= 0 or min(values) >= 0:
-        raise ValueError('IRR must have one positive and one negative value')
-
-    dates = [i*365 for i in range(len(values))]
-    max_epsilon = 1e-10  # Maximum epsilon for end of iteration
-    max_iterations = 50  # Maximum number of iterations
-
-    # Implement Newton's method
-    count_loop = True
-    iteration = 0
-    while iteration < max_iterations and count_loop:
-        result_value = irrResult(values, dates, guess_rate)
-        new_rate = guess_rate - result_value / irrResultDeriv(values, dates, guess_rate)
-        rate_delta = abs(new_rate - guess_rate)
-        guess_rate = new_rate
-        count_loop = (rate_delta > max_epsilon) and (abs(result_value) > max_epsilon)
-        iteration += 1
-
-    if count_loop:
-        raise ValueError('Unable to find valid IRR result.')
-
-    return guess_rate  # Return internal rate of return
 
 # Functions take input in the tokenized form fn(a, b, c)
 FUNCTION_MAP = {
